/***************************************************************************
                          csaveablelist.h  -  ancestor of alias/trigger handler
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne sep 8 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSAVEABLELIST_H
#define CSAVEABLELIST_H

#include <qobject.h>

#include "cactionbase.h"

#include "csaveablefield.h"

class KConfig;

/**
Abstract class that can load/save itself.
Its descendands will handle aliases/triggers.
  *@author Tomas Mecir
  */

class cSaveableList : public QObject, public cActionBase {
   Q_OBJECT
public: 
  /** constructor */
  cSaveableList (QString name, int sess);
  /** destructor */
  ~cSaveableList ();

  /** Saves the whole list to a configuration file. */
  void save ();

  /** go back to the first item */
  void reset () { cur = first; };
  /** advance list one item forwards */
  void operator++ (int) { cur = ((cur == NULL) ? NULL : cur->next); };
  /** advance list one item backwards */
  void operator-- (int) { cur = ((cur == NULL) ? NULL : cur->prev); };
  /** return current field */
  cSaveableField * operator* () { return cur; };
  /** Is the current item valid? False if we have run away from the list */
  operator bool () { return (cur != NULL); };

  /** is the list empty? */
  bool isEmpty () { return (_count == 0); };
  /** are we on the first item? */
  bool isFirst () { return (cur == first); };
  /** are we on the last item? */
  bool isLast () { return (cur == last); };
  /** returns count of items in the list */
  int count () { return _count; };

  /** sets marker to the current item */
  void setMarker () { marker = cur; };
  /** removes marker */
  void unsetMarker () { marker = NULL; };

  /** Adds a new item to the begin of the list. Returns true if successful. */
  bool addToBegin (cSaveableField *newitem);
  /** Adds a new item to the end of the list. Returns true if successful. */
  bool addToEnd (cSaveableField *newitem);
  /** Adds a new item after the current item. Returns true if successful. */
  bool addAfterCurrent (cSaveableField *newitem);
  /** Adds a new item after the marked item. Returns true if successful. */
  bool addAfterMarker (cSaveableField *newitem);
  /** Replaces current item with a new one. Returns true if successful. */
  bool replaceCurrent (cSaveableField *newitem);
  /** Replaces marked item with a new one. Returns true if successful. */
  bool replaceMarker (cSaveableField *newitem);
  /** moves current item by one item towards the begin of list */
  bool moveCurrentToFront ();
  /** moves current item by one item towards the end of list */
  bool moveCurrentToBack ();
  /** Removes first item from the list. Returns true if successful. */
  bool removeFirst (bool dontDelete = false);
  /** Removes last item from the list. Returns true if successful. */
  bool removeLast (bool dontDelete = false);
  /** Removes current item from the list. Returns true if successful. */
  bool removeCurrent (bool dontDelete = false);
  /** Removes marked item from the list. Returns true if successful. */
  bool removeMarked (bool dontDelete = false);
  /** clears the whole list */
  void clear (bool dontDelete = false);
  
  /** Prepares the object for a new string. Does nothing here; will probably
be redefined in child classes (to prevent recursive alias expansion, multiple
triggering of one trigger etc.). */
  virtual void newString () {};
  /** abstract; should process the command returned by matching cSaveableField
item; may emit a signal or whatever... */
  virtual void processCommands (const QStringList &commands) = 0;

  /** enable/disable aliases/triggers/whatever */
  void setEnabled (bool value) { _enabled = value; };
  /** am I enabled? */
  bool enabled () { return _enabled; };
protected:
  //LOAD/SAVE
  /** Loads the whole list from a configuration file. Function will be called
  from constructor of derived class */
  void load ();

  /** process recursion; does nothing here, redefine in childclasses if needed */
  virtual void processRecursion () {};
  
  /** config object must be created before calling load/save */
  KConfig *config;
  /** object name, e.g. "Trigger"; must be filled in by child class before calling load/save */
  QString objName;
  /** prototype object of the type stored in the list (cSaveableField descendant) */
  cSaveableField *prototype;
  
  /** required pointers to items */
  cSaveableField *first, *last, *cur, *marker;
  /** count of items is stored here */
  int _count;

  /** is processing enabled? */
  bool _enabled;

  /** was a recursion detected? */
  bool recursionAlert;
};

#endif
